"""initial

Revision ID: c6e034666a4b
Revises: 
Create Date: 2023-04-28 20:16:27.661742

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "c6e034666a4b"
down_revision = None
branch_labels = None
depends_on = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "EstimateType",
        sa.Column("name", sa.String(length=20), nullable=False),
        sa.Column("description", sa.Text(), nullable=False),
        sa.Column("relative", sa.Boolean(), nullable=False),
        sa.Column("default_shift_optimistic", sa.Float(), nullable=False),
        sa.Column("default_shift_pessimistic", sa.Float(), nullable=False),
        sa.Column("default_shift_average", sa.Float(), nullable=False),
        sa.Column("min_datapoints", sa.Integer(), nullable=False),
        sa.Column("max_datapoints", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )
    op.create_index(op.f("ix_EstimateType_id"), "EstimateType", ["id"], unique=False)
    op.create_table(
        "Project",
        sa.Column("name", sa.String(length=30), nullable=False),
        sa.Column("color", sa.String(length=10), nullable=False),
        sa.Column("description", sa.Text(), nullable=False),
        sa.Column("parent_id", sa.Integer(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["parent_id"], ["Project.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_Project_id"), "Project", ["id"], unique=False)
    op.create_index(op.f("ix_Project_parent_id"), "Project", ["parent_id"], unique=False)
    op.create_table(
        "Role",
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("permissions", sa.types.BigInteger(), nullable=False),
        sa.Column(
            "role_type", sa.Enum("GLOBAL", "TEAM", "PROJECT", name="roletype"), nullable=False
        ),
        sa.Column("builtin", sa.Boolean(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_Role_id"), "Role", ["id"], unique=False)
    op.create_index(op.f("ix_Role_name"), "Role", ["name"], unique=True)
    op.create_index(op.f("ix_Role_permissions"), "Role", ["permissions"], unique=True)
    op.create_table(
        "Team",
        sa.Column("name", sa.String(length=30), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_Team_id"), "Team", ["id"], unique=False)
    op.create_table(
        "Timesink",
        sa.Column("name", sa.String(length=30), nullable=False),
        sa.Column("color", sa.String(length=10), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )
    op.create_index(op.f("ix_Timesink_id"), "Timesink", ["id"], unique=False)
    op.create_table(
        "User",
        sa.Column("name", sa.String(length=20), nullable=False),
        sa.Column("full_name", sa.String(length=50), nullable=False),
        sa.Column("password", sa.BINARY(length=60), nullable=False),
        sa.Column("avatar", sa.Text(), nullable=False),
        sa.Column("session_id", sa.String(length=60), nullable=True),
        sa.Column("interrupted_task_id", sa.Integer(), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["interrupted_task_id"], ["Task.id"], use_alter=True),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_User_full_name"), "User", ["full_name"], unique=False)
    op.create_index(op.f("ix_User_id"), "User", ["id"], unique=False)
    op.create_index(
        op.f("ix_User_interrupted_task_id"), "User", ["interrupted_task_id"], unique=False
    )
    op.create_index(op.f("ix_User_name"), "User", ["name"], unique=True)
    op.create_index(op.f("ix_User_session_id"), "User", ["session_id"], unique=False)
    op.create_table(
        "AssignedGlobalRole",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("role_id", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["role_id"], ["Role.id"],),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_AssignedGlobalRole_id"), "AssignedGlobalRole", ["id"], unique=False)
    op.create_index(
        op.f("ix_AssignedGlobalRole_role_id"), "AssignedGlobalRole", ["role_id"], unique=False
    )
    op.create_index(
        op.f("ix_AssignedGlobalRole_user_id"), "AssignedGlobalRole", ["user_id"], unique=False
    )
    op.create_table(
        "AssignedProjectRole",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("role_id", sa.Integer(), nullable=False),
        sa.Column("project_id", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["project_id"], ["Project.id"],),
        sa.ForeignKeyConstraint(["role_id"], ["Role.id"],),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_AssignedProjectRole_id"), "AssignedProjectRole", ["id"], unique=False)
    op.create_index(
        op.f("ix_AssignedProjectRole_project_id"),
        "AssignedProjectRole",
        ["project_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_AssignedProjectRole_role_id"), "AssignedProjectRole", ["role_id"], unique=False
    )
    op.create_index(
        op.f("ix_AssignedProjectRole_user_id"), "AssignedProjectRole", ["user_id"], unique=False
    )
    op.create_table(
        "AssignedTeamRole",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("role_id", sa.Integer(), nullable=False),
        sa.Column("team_id", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["role_id"], ["Role.id"],),
        sa.ForeignKeyConstraint(["team_id"], ["Team.id"],),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_AssignedTeamRole_id"), "AssignedTeamRole", ["id"], unique=False)
    op.create_index(
        op.f("ix_AssignedTeamRole_role_id"), "AssignedTeamRole", ["role_id"], unique=False
    )
    op.create_index(
        op.f("ix_AssignedTeamRole_team_id"), "AssignedTeamRole", ["team_id"], unique=False
    )
    op.create_index(
        op.f("ix_AssignedTeamRole_user_id"), "AssignedTeamRole", ["user_id"], unique=False
    )
    op.create_table(
        "EstimateStatistics",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("estimate_type_id", sa.Integer(), nullable=False),
        sa.Column("num_datapoints", sa.Integer(), nullable=False),
        sa.Column("evaluated", sa.DateTime(), nullable=False),
        sa.Column("shift_optimistic", sa.Float(), nullable=False),
        sa.Column("shift_pessimistic", sa.Float(), nullable=False),
        sa.Column("shift_average", sa.Float(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["estimate_type_id"], ["EstimateType.id"],),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_EstimateStatistics_estimate_type_id"),
        "EstimateStatistics",
        ["estimate_type_id"],
        unique=False,
    )
    op.create_index(op.f("ix_EstimateStatistics_id"), "EstimateStatistics", ["id"], unique=False)
    op.create_index(
        op.f("ix_EstimateStatistics_user_id"), "EstimateStatistics", ["user_id"], unique=False
    )
    op.create_table(
        "Task",
        sa.Column("name", sa.String(length=50), nullable=False),
        sa.Column("description", sa.Text(), nullable=False),
        sa.Column("project_id", sa.Integer(), nullable=False),
        sa.Column("parent_id", sa.Integer(), nullable=True),
        sa.Column("order_id", sa.Integer(), nullable=True),
        sa.Column(
            "task_state",
            sa.Enum(
                "REQUEST",
                "PLANNING",
                "DEFERRED",
                "SCHEDULED",
                "IN_PROGRESS",
                "DONE",
                "DISCARDED",
                "HOLD",
                name="taskstate",
            ),
            nullable=False,
        ),
        sa.Column("task_type", sa.Enum("FEATURE", "BUG", "ADHOC", name="tasktype"), nullable=False),
        sa.Column(
            "task_priority",
            sa.Enum("VERY_HIGH", "HIGH", "MEDIUM", "LOW", "VERY_LOW", name="taskpriority"),
            nullable=False,
        ),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("finished_at", sa.DateTime(), nullable=True),
        sa.Column("scheduled_average_start", sa.DateTime(), nullable=True),
        sa.Column("scheduled_average_end", sa.DateTime(), nullable=True),
        sa.Column("scheduled_optimistic_start", sa.DateTime(), nullable=True),
        sa.Column("scheduled_optimistic_end", sa.DateTime(), nullable=True),
        sa.Column("scheduled_pessimistic_start", sa.DateTime(), nullable=True),
        sa.Column("scheduled_pessimistic_end", sa.DateTime(), nullable=True),
        sa.Column("scheduled_assignee_id", sa.Integer(), nullable=True),
        sa.Column("automatic_schedule", sa.Boolean(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["parent_id"], ["Task.id"],),
        sa.ForeignKeyConstraint(["project_id"], ["Project.id"],),
        sa.ForeignKeyConstraint(["scheduled_assignee_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_Task_id"), "Task", ["id"], unique=False)
    op.create_index(op.f("ix_Task_parent_id"), "Task", ["parent_id"], unique=False)
    op.create_index(op.f("ix_Task_project_id"), "Task", ["project_id"], unique=False)
    op.create_index(
        op.f("ix_Task_scheduled_assignee_id"), "Task", ["scheduled_assignee_id"], unique=False
    )
    op.create_table(
        "WorkPeriod",
        sa.Column("cls_name", sa.String(length=20), nullable=False),
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("started_at", sa.DateTime(), nullable=False),
        sa.Column("ended_at", sa.DateTime(), nullable=True),
        sa.Column("comment", sa.Text(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_WorkPeriod_id"), "WorkPeriod", ["id"], unique=False)
    op.create_index(op.f("ix_WorkPeriod_user_id"), "WorkPeriod", ["user_id"], unique=False)
    op.create_table(
        "assoc_project_team",
        sa.Column("project_id", sa.Integer(), nullable=False),
        sa.Column("team_id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["project_id"], ["Project.id"],),
        sa.ForeignKeyConstraint(["team_id"], ["Team.id"],),
        sa.PrimaryKeyConstraint("project_id", "team_id"),
    )
    op.create_table(
        "Estimate",
        sa.Column("task_id", sa.Integer(), nullable=False),
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("created_at", sa.DateTime(), nullable=False),
        sa.Column("estimate_type_id", sa.Integer(), nullable=False),
        sa.Column("estimated_duration", sa.Interval(), nullable=False),
        sa.Column("expectation_optimistic", sa.Interval(), nullable=False),
        sa.Column("expectation_pessimistic", sa.Interval(), nullable=False),
        sa.Column("expectation_average", sa.Interval(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["estimate_type_id"], ["EstimateType.id"],),
        sa.ForeignKeyConstraint(["task_id"], ["Task.id"],),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_Estimate_estimate_type_id"), "Estimate", ["estimate_type_id"], unique=False
    )
    op.create_index(op.f("ix_Estimate_id"), "Estimate", ["id"], unique=False)
    op.create_index(op.f("ix_Estimate_task_id"), "Estimate", ["task_id"], unique=False)
    op.create_index(op.f("ix_Estimate_user_id"), "Estimate", ["user_id"], unique=False)
    op.create_table(
        "WorkPeriodNotWorking",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["WorkPeriod.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(
        op.f("ix_WorkPeriodNotWorking_id"), "WorkPeriodNotWorking", ["id"], unique=False
    )
    op.create_table(
        "WorkPeriodTask",
        sa.Column("task_id", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["WorkPeriod.id"],),
        sa.ForeignKeyConstraint(["task_id"], ["Task.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_WorkPeriodTask_id"), "WorkPeriodTask", ["id"], unique=False)
    op.create_index(op.f("ix_WorkPeriodTask_task_id"), "WorkPeriodTask", ["task_id"], unique=False)
    op.create_table(
        "WorkPeriodTimesink",
        sa.Column("timesink_id", sa.Integer(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["WorkPeriod.id"],),
        sa.ForeignKeyConstraint(["timesink_id"], ["Timesink.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_WorkPeriodTimesink_id"), "WorkPeriodTimesink", ["id"], unique=False)
    op.create_index(
        op.f("ix_WorkPeriodTimesink_timesink_id"),
        "WorkPeriodTimesink",
        ["timesink_id"],
        unique=False,
    )
    op.create_table(
        "WorkPeriodWorking",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["WorkPeriod.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_WorkPeriodWorking_id"), "WorkPeriodWorking", ["id"], unique=False)
    op.create_table(
        "TaskHistory",
        sa.Column("date", sa.DateTime(), nullable=False),
        sa.Column("task_id", sa.Integer(), nullable=False),
        sa.Column("scheduled_average_end", sa.DateTime(), nullable=False),
        sa.Column("scheduled_optimistic_end", sa.DateTime(), nullable=False),
        sa.Column("scheduled_pessimistic_end", sa.DateTime(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["task_id"], ["Task.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_TaskHistory_id"), "TaskHistory", ["id"], unique=False)
    op.create_index(op.f("ix_TaskHistory_task_id"), "TaskHistory", ["task_id"], unique=False)
    op.create_table(
        "RRuleDate",
        sa.Column(
            "freq",
            sa.Enum("WEEKLY", "DAILY", "MONTHLY", "YEARLY", name="frequency"),
            nullable=False,
        ),
        sa.Column("byweekday", sa.Integer(), nullable=False),
        sa.Column("interval", sa.Integer(), nullable=False),
        sa.Column("dtstart", sa.Date(), nullable=False),
        sa.Column("until", sa.Date(), nullable=True),
        sa.Column("cls_name", sa.String(length=20), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_RRuleDate_id"), "RRuleDate", ["id"], unique=False)
    op.create_table(
        "Absence",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("reason", sa.String(length=50), nullable=True),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["RRuleDate.id"],),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_Absence_id"), "Absence", ["id"], unique=False)
    op.create_index(op.f("ix_Absence_user_id"), "Absence", ["user_id"], unique=False)
    op.create_table(
        "ScheduleRule",
        sa.Column("user_id", sa.Integer(), nullable=False),
        sa.Column("timezone", sa.String(length=30), nullable=False),
        sa.Column("start", sa.Time(), nullable=False),
        sa.Column("end", sa.Time(), nullable=False),
        sa.Column("id", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(["id"], ["RRuleDate.id"],),
        sa.ForeignKeyConstraint(["user_id"], ["User.id"],),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_index(op.f("ix_ScheduleRule_id"), "ScheduleRule", ["id"], unique=False)
    op.create_index(op.f("ix_ScheduleRule_user_id"), "ScheduleRule", ["user_id"], unique=False)
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_ScheduleRule_user_id"), table_name="ScheduleRule")
    op.drop_index(op.f("ix_ScheduleRule_id"), table_name="ScheduleRule")
    op.drop_table("ScheduleRule")
    op.drop_index(op.f("ix_Absence_user_id"), table_name="Absence")
    op.drop_index(op.f("ix_Absence_id"), table_name="Absence")
    op.drop_table("Absence")
    op.drop_index(op.f("ix_RRuleDate_id"), table_name="RRuleDate")
    op.drop_table("RRuleDate")
    op.drop_index(op.f("ix_TaskHistory_task_id"), table_name="TaskHistory")
    op.drop_index(op.f("ix_TaskHistory_id"), table_name="TaskHistory")
    op.drop_table("TaskHistory")
    op.drop_index(op.f("ix_WorkPeriodWorking_id"), table_name="WorkPeriodWorking")
    op.drop_table("WorkPeriodWorking")
    op.drop_index(op.f("ix_WorkPeriodTimesink_timesink_id"), table_name="WorkPeriodTimesink")
    op.drop_index(op.f("ix_WorkPeriodTimesink_id"), table_name="WorkPeriodTimesink")
    op.drop_table("WorkPeriodTimesink")
    op.drop_index(op.f("ix_WorkPeriodTask_task_id"), table_name="WorkPeriodTask")
    op.drop_index(op.f("ix_WorkPeriodTask_id"), table_name="WorkPeriodTask")
    op.drop_table("WorkPeriodTask")
    op.drop_index(op.f("ix_WorkPeriodNotWorking_id"), table_name="WorkPeriodNotWorking")
    op.drop_table("WorkPeriodNotWorking")
    op.drop_index(op.f("ix_Estimate_user_id"), table_name="Estimate")
    op.drop_index(op.f("ix_Estimate_task_id"), table_name="Estimate")
    op.drop_index(op.f("ix_Estimate_id"), table_name="Estimate")
    op.drop_index(op.f("ix_Estimate_estimate_type_id"), table_name="Estimate")
    op.drop_table("Estimate")
    op.drop_table("assoc_project_team")
    op.drop_index(op.f("ix_WorkPeriod_user_id"), table_name="WorkPeriod")
    op.drop_index(op.f("ix_WorkPeriod_id"), table_name="WorkPeriod")
    op.drop_table("WorkPeriod")
    op.drop_index(op.f("ix_Task_scheduled_assignee_id"), table_name="Task")
    op.drop_index(op.f("ix_Task_project_id"), table_name="Task")
    op.drop_index(op.f("ix_Task_parent_id"), table_name="Task")
    op.drop_index(op.f("ix_Task_id"), table_name="Task")
    op.drop_table("Task")
    op.drop_index(op.f("ix_EstimateStatistics_user_id"), table_name="EstimateStatistics")
    op.drop_index(op.f("ix_EstimateStatistics_id"), table_name="EstimateStatistics")
    op.drop_index(op.f("ix_EstimateStatistics_estimate_type_id"), table_name="EstimateStatistics")
    op.drop_table("EstimateStatistics")
    op.drop_index(op.f("ix_AssignedTeamRole_user_id"), table_name="AssignedTeamRole")
    op.drop_index(op.f("ix_AssignedTeamRole_team_id"), table_name="AssignedTeamRole")
    op.drop_index(op.f("ix_AssignedTeamRole_role_id"), table_name="AssignedTeamRole")
    op.drop_index(op.f("ix_AssignedTeamRole_id"), table_name="AssignedTeamRole")
    op.drop_table("AssignedTeamRole")
    op.drop_index(op.f("ix_AssignedProjectRole_user_id"), table_name="AssignedProjectRole")
    op.drop_index(op.f("ix_AssignedProjectRole_role_id"), table_name="AssignedProjectRole")
    op.drop_index(op.f("ix_AssignedProjectRole_project_id"), table_name="AssignedProjectRole")
    op.drop_index(op.f("ix_AssignedProjectRole_id"), table_name="AssignedProjectRole")
    op.drop_table("AssignedProjectRole")
    op.drop_index(op.f("ix_AssignedGlobalRole_user_id"), table_name="AssignedGlobalRole")
    op.drop_index(op.f("ix_AssignedGlobalRole_role_id"), table_name="AssignedGlobalRole")
    op.drop_index(op.f("ix_AssignedGlobalRole_id"), table_name="AssignedGlobalRole")
    op.drop_table("AssignedGlobalRole")
    op.drop_index(op.f("ix_User_session_id"), table_name="User")
    op.drop_index(op.f("ix_User_name"), table_name="User")
    op.drop_index(op.f("ix_User_interrupted_task_id"), table_name="User")
    op.drop_index(op.f("ix_User_id"), table_name="User")
    op.drop_index(op.f("ix_User_full_name"), table_name="User")
    op.drop_table("User")
    op.drop_index(op.f("ix_Timesink_id"), table_name="Timesink")
    op.drop_table("Timesink")
    op.drop_index(op.f("ix_Team_id"), table_name="Team")
    op.drop_table("Team")
    op.drop_index(op.f("ix_Role_permissions"), table_name="Role")
    op.drop_index(op.f("ix_Role_name"), table_name="Role")
    op.drop_index(op.f("ix_Role_id"), table_name="Role")
    op.drop_table("Role")
    op.drop_index(op.f("ix_Project_parent_id"), table_name="Project")
    op.drop_index(op.f("ix_Project_id"), table_name="Project")
    op.drop_table("Project")
    op.drop_index(op.f("ix_EstimateType_id"), table_name="EstimateType")
    op.drop_table("EstimateType")
    # ### end Alembic commands ###
